//-------------------------------------------------------------------
//	File:		   dialog.c
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "dialog.h"

#define labelXOffset 150
#define valueXOffset 15
#define textshift 15

//--------------------------------------------------------------------------------------

Widget::Widget() {
	size=10;
	return;
};
Widget::~Widget() {};

Texture *Widget::tex=0;
int   Widget::textSize=10;
Text *Widget::text=0;

//--------------------------------------------------------------------------------------

void Widget::init(char *s, int inID, int inindex, int intype,
		      float inx, float iny, float inwidth, float inheight,
			  float invalue, float inmin, float inmax, int flag, int linear) {
	
	ID=inID;
	size=textSize;
	index=inindex;
	type=intype;
	x=inx;
	y=iny;
	width=inwidth;
	height=inheight;
	x2=x+width;
	y2=y+height;
	strcpy(label,s);
}

//------------------------------------------------------------

int Widget::inside(int mousex, int mousey, int ox, int oy) {
	int flag=0;

	if ((mousex>=(x+ox))&&(mousex<=(x2+ox))) flag =1;
	if ((mousey>=(y+oy))&&(mousey<=(y2+oy))) flag|=2;
	return flag;
}

//--------------------------------------------------------------------------------------

void Widget::show(float ox,float oy, int current){
	return;
}

//--------------------------------------------------------------------------------------

int Widget::process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current) {
	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

Slider::Slider() {
	texX0=0;
	texY0=.75;
	texX1=.25;
	texY1=1;
	linear=0;
	return;
};
Slider::~Slider() {};

float Slider::texX0=0;
float Slider::texY0=0;
float Slider::texX1=1;
float Slider::texY1=1;

//--------------------------------------------------------------------------------------

void Slider::init(char *s, int inID, int inindex, int intype,
		      float inx, float iny, float inwidth, float inheight,
			   float invalue, float inmin, float inmax, int flag, int inlinear) {

	this->Widget::init(s, inID, inindex,intype,inx, iny, inwidth, inheight);

	truncate=flag;
	min=inmin;
	max=inmax;
	value=invalue;
	range=max-min;
	linear=inlinear;
}

//------------------------------------------------------------

void Slider::show(float ox,float oy, int current){
	float f,v;
	char s[80];

	f=(value-min)/range;
	if (!linear) f=sqrt(f);
	v=(f*width),
	drawQ(tex, x+ox,    y+oy+6, x2+ox,     y2+oy-6, texX0+.1 ,texY0, texX1-.1, texY1);
	drawQ(tex, x+ox+v-10,y+oy, x +ox+v+10, y2+oy,   texX0+.25,texY0, texX1+.25,texY1);
	if (truncate) sprintf(s,"%5d",(int)value);
	else          sprintf(s,"%5.2f",   value);

	text->print(label, x2+labelXOffset*textSize/20.+ox, y+oy+textshift,textSize);
	text->print(s    , x2+valueXOffset             +ox, y+oy+textshift,textSize);
	return;
}

//--------------------------------------------------------------------------------------

int Slider::process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current) {
	if (mouse->lbutton) {
		switch(mouse->lbutton) {
			float f;
		case 1:
			if (inside(mouse->x, mouse->y, ox,oy)==3) {
				f=(mouse->x-ox-x)/width;
				if (!linear) value=pow(f,2)*range+min;
				else         value=    f   *range+min;
				*outValue=value;
				return 1;
			}
			break;
		case 2:
			if ((inside(mouse->x, mouse->y, ox,oy)&1)&&current) {
				f=(mouse->x-ox-x)/width;
				if (!linear) value=pow(f,2)*range+min;
				else         value=    f   *range+min;
				*outValue=value;
				return 1;
			}
			break;
		}
	}
	if (key->skey&&current) {
		if (key->skey==GLUT_KEY_LEFT) {
			if (truncate) value-=1;
				else value-=.01*range;
			if (value<min) value=min;
			*outValue=value;
			return 1;
		}
		if (key->skey==GLUT_KEY_RIGHT) {
			if (truncate) value+=1;
			else value+=.01*range;
			if (value>max) value=max;
			*outValue=value;
			return 1;
		}
	}
	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

Toggle::Toggle() {
	texX0=.5;
	texY0=.75;
	texX1=.75;
	texY1=1;
	return;
};
Toggle::~Toggle() {};

float Toggle::texX0=0;
float Toggle::texY0=0;
float Toggle::texX1=1;
float Toggle::texY1=1;

//--------------------------------------------------------------------------------------

void Toggle::init(char *s, int inID, int inindex, int intype,
		      float inx, float iny, float inwidth, float inheight,
			   float invalue, float inmin, float inmax, int flag, int linear) {

	this->Widget::init(s, inID, inindex,intype, inx, iny, inwidth, inheight);
	on=invalue;
}

//---------------------------------------------------------------------------

void Toggle::show(float ox,float oy, int incurrent){
	float current=incurrent*.33+.66;
	if (on) {
		text->print("On",  x2+valueXOffset+ox+10, y+oy+textshift,textSize);
		glColor3f(.5*current*text->inten,1*current*text->inten,.5*current*text->inten);
	}
	else {
		text->print("Off", x2+valueXOffset+ox+10, y+oy+textshift,textSize);
		glColor3f(1*current*text->inten,.5*current*text->inten,.5*current*text->inten);
	}
	drawQ(tex, x+ox,y+oy, x2+ox,y2+oy, (on)?texX0+.25:texX0,texY0,
		                                (on)?texX1+.25:texX1,texY1);
	text->print(label, x2+(labelXOffset)*textSize/30.+20+ox, y+oy+textshift,textSize);
	return;
}

//--------------------------------------------------------------------------------------

int Toggle::process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current) {
	if (mouse->lbutton) {
		switch(mouse->lbutton) {
		case 1:
			if (inside(mouse->x, mouse->y, ox,oy)==3) {
				on=!on;
				*outValue=on;
				return 1;
			}
			break;
		}
	}
	if ((key->key||key->skey)&&current) {
		if ((key->key==13)||
			 (key->skey==GLUT_KEY_LEFT)||(key->skey==GLUT_KEY_RIGHT)) {
			on=!on;
			*outValue=on;
			return 1;
		}
	}
	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

Button::Button() {
	texX0=0;
	texY0=.5;
	texX1=.25;
	texY1=.75;
	return;
};
Button::~Button() {};

float Button::texX0=0;
float Button::texY0=0;
float Button::texX1=1;
float Button::texY1=1;

//--------------------------------------------------------------------------------------

void Button::init(char *s, int inID, int inindex, int intype,
		      float inx, float iny, float inwidth, float inheight,
			   float invalue, float inmin, float inmax, int flag, int linear) {

	this->Widget::init(s, inID, inindex,intype,inx, iny, inwidth, inheight);
	on=0;
}

//--------------------------------------------------------------------------------------

void Button::show(float ox,float oy, int current){
	drawQ(tex, x+ox,y+oy, x2+ox,y2+oy,(on)?texX0+.25:texX0,texY0,
		                               (on)?texX1+.25:texX1,texY1);
	text->print(label, x2+valueXOffset+ox, y+oy+textshift,textSize);
	return;
}

//--------------------------------------------------------------------------------------

int Button::process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current) {
	if (mouse->lbutton) {
		switch(mouse->lbutton) {
		case 1: if  (inside(mouse->x, mouse->y, ox,oy)==3) {on=1; return 2;}
		case 2: if (current) {
			if ((inside(mouse->x, mouse->y, ox,oy)==3)&&current) on=1;
			else on=0;
				  }
			break;
		case 3:
			if ((inside(mouse->x, mouse->y, ox,oy)==3)&&current) {
				on=1;
				return 1;
			}
			break;
		}
	}
	if ((key->key||key->skey)&&current) {
		if ((key->key==13)||(key->skey==GLUT_KEY_LEFT)||(key->skey==GLUT_KEY_RIGHT)) {
			return 1;
		}
	}
	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

Back::Back() {
	texX0=0;
	texY0=.25;
	texX1=.25;
	texY1=.5;
	return;
};
Back::~Back() {};

float Back::texX0=0;
float Back::texY0=0;
float Back::texX1=1;
float Back::texY1=1;

//--------------------------------------------------------------------------------------

void Back::init(char *s, int inID, int inindex, int intype,
		      float inx, float iny, float inwidth, float inheight,
			   float invalue, float inmin, float inmax, int flag, int linear) {

	this->Widget::init(s, inID, inindex,intype,inx, iny, inwidth, inheight);
	on=0;
}

//--------------------------------------------------------------------------------------

void Back::show(float ox,float oy, int current){
	drawQ(tex, x+ox,y+oy, x2+ox,y2+oy,(on)?texX0+.25:texX0,texY0,
		                               (on)?texX1+.25:texX1,texY1);
	text->print(label, x2+valueXOffset+ox, y+oy+textshift,textSize);
	return;
}

//--------------------------------------------------------------------------------------

int Back::process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current) {
	if (mouse->lbutton) {
		switch(mouse->lbutton) {
		case 1: on=1; return 2;
		case 2: if ((inside(mouse->x, mouse->y, ox,oy)!=3)) on=0; else on=1;
			break;
		case 3:
			if ((inside(mouse->x, mouse->y, ox,oy)==3)&&current) return 1;
			break;
		}
	}
	if ((key->key||key->skey)&&current) {
		if ((key->key==13)||(key->skey==GLUT_KEY_LEFT)||(key->skey==GLUT_KEY_RIGHT)) {
			return 1;
		}
	}
	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

Title::Title() {
	texX0=.5;
	texY0=.5;
	texX1=.75;
	texY1=.75;
	return;
};
Title::~Title() {};

float Title::texX0=0;
float Title::texY0=0;
float Title::texX1=1;
float Title::texY1=1;

//--------------------------------------------------------------------------------------

void Title::init(char *s, int inID, int inindex, int intype,
		      float inx, float iny, float inwidth, float inheight,
			   float invalue, float inmin, float inmax, int flag, int linear) {
	
	this->Widget::init(s, inID, inindex,intype,inx, iny, inwidth, inheight);
}

//--------------------------------------------------------------------------------------

void Title::show(float ox,float oy, int current){
	drawQ(tex, x+ox,y+oy, x2+ox,y2+oy, texX0,texY0, texX1,texY1);
	text->print(label, x2+valueXOffset+ox, y+oy+textshift,textSize);
	return;
}

//--------------------------------------------------------------------------------------

int Title::process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current) {
	if (mouse->lbutton) {
		switch(mouse->lbutton) {
		case 1:
			if (inside(mouse->x, mouse->y, ox,oy)==3) {on=1; return 3;}
			break;
		case 2:
			if (current&&on) return 3;
			break;
		case 3:
			if (current) {on=0; return 3;}
			break;
		}
	}
	if (key->skey&&current) {
		if ((key->skey==GLUT_KEY_LEFT)   ||(key->skey==GLUT_KEY_RIGHT)||
		    (key->skey==GLUT_KEY_PAGE_UP)||(key->skey==GLUT_KEY_PAGE_DOWN))  return 3;
	}
	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

Dialog::Dialog() {
	ndialogPushed=0;
	clear();
	saveView=0;
	fadedir=0; fade=0;
	mouse=0;	key=0; text=0; time=0;
	return;
};

Dialog::~Dialog() {
	for (int i=0; i<nitems; i++) if (controls[i]) delete controls[i];
};

//--------------------------------------------------------------------------------------

void Dialog::clear(void) {
	ID=-1;
	nitems=0;
	activeID=-1;
	currentIndex=0;
	activeValue=0;
	ox=0; oy=0;
	for (int i=0; i<maxa; i++) controls[i]=0;
	return;
}

//--------------------------------------------------------------------------------------

int Dialog::control(int intype, char *inlabel, int inID,
						   int inx,int iny, int inwidth,int inheight,
						   float invalue, float inmin, float inmax, int flag, int linear) {
	int found=-1;

	for (int i=0; i<nitems; i++) {
		if (inID==controls[i]->ID) {
			found=i;
			break;
		}
	}
	if (found<0) {
		if (nitems<maxa) {
			found=nitems;
			nitems++;
		}
	}
	if (found<0) return found;
	switch(intype) {
	case SLIDER:
		if (controls[found]==0) controls[found]=new Slider;
		if (controls[found]==0) {nitems-=1; return -1;}
		controls[found]->init(inlabel,inID,found,SLIDER,inx,iny,inwidth,inheight,invalue,inmin,inmax,flag, linear);
		break;

	case TOGGLE:
		if (controls[found]==0) controls[found]=new Toggle;
		if (controls[found]==0) {nitems-=1; return -1;}
		controls[found]->init(inlabel,inID,found,TOGGLE,inx,iny,inwidth,inheight,invalue,0,0,0);
		break;

	case BUTTON:
		if (controls[found]==0) controls[found]=new Button;
		if (controls[found]==0) {nitems-=1; return -1;}
		controls[found]->init(inlabel,inID,found,BUTTON,inx,iny,inwidth,inheight,0,0,0,0);
		break;

	case BACK:
		if (controls[found]==0) controls[found]=new Back;
		if (controls[found]==0) {nitems-=1; return -1;}
		controls[found]->init(inlabel,inID,found,BACK,inx,iny,inwidth,inheight,0,0,0,0);
		break;

	case TITLE:
		if (controls[found]==0) controls[found]=new Title;
		if (controls[found]==0) {nitems-=1; return -1;}
		controls[found]->init(inlabel,inID,found,TITLE, inx,iny,inwidth,inheight,0,0,0,0);
		break;
	}
	return found;
}

//--------------------------------------------------------------------------------------

void Dialog::show(void) {
	float lastox;

	if (fade!=fadedir) {
		fade+=time->fDeltaTime*3*(fadedir*2-1);
		fade=(fade<0)?0:(fade>1)?1:fade;
	}
	lastox=ox;
	inten=fade;
	text->inten=inten;
	for (int i=0; i<nitems; i++) {
		if (i==currentIndex) glColor3f(    inten,    inten,    inten);
		else                 glColor3f(.75*inten,.75*inten,.75*inten);
		controls[i]->show(ox-300*pow(1-fade,2),oy, (currentIndex==i));
	}
	ox=lastox;
	return;
}

//--------------------------------------------------------------------------

void Dialog::check(void) {
	float value=0;
	int status=0;
	
	activeID=-1;
	activeValue=0;
	for (int i=0; i<nitems; i++) {
		status=controls[i]->process(key, mouse, ox, oy, &value, (currentIndex==i));
		if (status) {
			if (mouse->lbutton==1) currentIndex=i;
			if (status==1) {
				activeID=controls[i]->ID;
				activeValue=value;
				return;
			}
			if (status==3) {
				float dragx1, dragy1;
				static float dragx0=0,dragy0=0, lastox=0, lastoy=0;
				switch(mouse->lbutton) {
				case 1:
					dragx0=mouse->x;
					dragy0=mouse->y;
					break;
				case 2:
					dragx1=mouse->x;
					dragy1=mouse->y;
					ox=lastox+dragx1-dragx0;
					oy=lastoy+dragy1-dragy0;
					break;
				case 3:
					lastox=ox;
					lastoy=oy;
					break;
				}
				switch(key->skey) {
				case GLUT_KEY_LEFT:  ox-=2; break;
				case GLUT_KEY_RIGHT: ox+=2; break;
				case GLUT_KEY_PAGE_UP:   oy+=2; break;
				case GLUT_KEY_PAGE_DOWN: oy-=2; break;
				}
			break;
			}
		}
	}
	return;
}

//----------------------------------------------------------------------------

void Dialog::service(void) {
	if (key->skey==GLUT_KEY_UP)
		currentIndex=(currentIndex==0)?nitems-1:currentIndex-1;

	if ((key->key==9)||(key->skey==GLUT_KEY_DOWN))
		currentIndex=(currentIndex>=(nitems-1))? 0: currentIndex+1;

	show();
	check();
	return;
}

//--------------------------------------------------------------------------------------

void Dialog::init(int ID, int inpush) {
	float cox=ox;
	float coy=oy;
	if (inpush) push();
	dialog_init(ID,1);
	ox=cox;
	oy=coy;
	return;
}

//--------------------------------------------------------------------

void Dialog::push(void) {
	if (ndialogPushed<maxDialogDepth) dialogStack[ndialogPushed++]=ID;
	else log_message("** Attempt to push dialog beyond limits");
	return;
}

//--------------------------------------------------------------------

void Dialog::pop(void) {
	if (ndialogPushed>0) {
		dialog_init(dialogStack[--ndialogPushed],1);
	} else log_message("** Attempt to pop no dialog");
	return;
}

//--------------------------------------------------------------------

void Dialog::run(void) {
	service();
	if (activeID > -1) dialog_do();
}

//--------------------------------------------------------------------------


//--------------------------------------------------------------------
//--------------------------------------------------------------------


void drawQ(Texture *tex, float x, float y, float x2, float y2,
		   float texX0,float texY0, float texX1, float texY1) {
	glAlphaFunc(GL_GREATER,.05);
	glEnable(GL_ALPHA_TEST);
	glDepthMask(GL_FALSE);
	//glEnable(GL_DEPTH_TEST);
	if (tex) {
		glEnable(GL_TEXTURE_2D);
		tex->bind();
	}
	glPushAttrib(GL_POLYGON_BIT);
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glBegin(GL_QUADS);
			glTexCoord2f(texX0,texY0); glVertex2f(x ,y);
			glTexCoord2f(texX1,texY0); glVertex2f(x2,y);
			glTexCoord2f(texX1,texY1); glVertex2f(x2,y2);
			glTexCoord2f(texX0,texY1); glVertex2f(x ,y2);
			glEnd();
		glPopAttrib();
	glDisable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER,0);
	//glDisable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	if (tex) glDisable(GL_TEXTURE_2D);
	return;
}

//-----------------------------------------------------------
