// mesh.cpp

#include "mesh.h"
#include "vector.h"

//------------------------------------------------------------------------------------

Mesh::Mesh() {
	size=1000; sizex=size; sizey=size;
	resx=32; resy=32;
	alphatest=0;
};

Mesh::~Mesh() {
};

//------------------------------------------------------------------------------------

void Mesh::init(Bump *bump) {
	int ix,iy;
	float qx=0,qz=0, fx,fy, tx,ty;

	qx=0;
	qz=0;
	for     (iy=0; iy<=resy; iy++) {
		ty=(float)iy/resy;
		fy=ty-.5;
		for (ix=0; ix<=resx; ix++) { 
			tx=(float)ix/resx;
			fx=tx-.5;
			data[ix][iy][0]=fx*sizex-qx;
			data[ix][iy][1]=bump->get(tx,ty)*hw->pref->page;
			data[ix][iy][2]=fy*sizey-qz;
			data[ix][iy][3]=0;
			data[ix][iy][4]=1;
			data[ix][iy][5]=0;
		}
	}
	calc_normal();
	//calc_flat();
	return;
}

//------------------------------------------------------------------------------------

void Mesh::init(void) {
	int ix,iy;
	float sizexinc,sizeyinc, tx,ty, qx=0,qy=0,fx,fy;

	sizexinc=sizex/(resx-1);
	sizeyinc=sizey/(resy-1);
	qx=.25*sizexinc;
	qy=.25*sizeyinc;
	for    (iy=0; iy<=resy; iy++) {
		ty=(float)iy/resy;
		fy=ty-.5;
		for (ix=0; ix<=resx; ix++) {
			tx=(float)ix/resx;
			fx=tx-.5;
			data[ix][iy][0]=ix*sizexinc-.5*sizex-qx;
			data[ix][iy][1]=y;
			data[ix][iy][2]=iy*sizeyinc-.5*sizey-qy;
			data[ix][iy][3]=0;
			data[ix][iy][4]=1;
			data[ix][iy][5]=0;
		}
	}
	//calc_normal();
	//calc_flat();
	return;
}

//------------------------------------------------------------------------------------

void Mesh::draw(Pref *pref) {
	int ix,iy,iy2;
	float tx,ty,ty2, resxtexmag, resytexmag;

	glPushMatrix();
		glTranslatef(x,y,z);
		resxtexmag=(resx-1)/texmagx;
		resytexmag=(resy-1)/texmagy;
		set(pref);
		for       (iy=0; iy<resy; iy++) {
			iy2=iy+1;
			ty =(float)iy /resytexmag+texOffsety;
			ty2=(float)iy2/resytexmag+texOffsety;
			glBegin(GL_TRIANGLE_STRIP);
				for (ix=0; ix<=resx; ix++) {
					if (data[ix][iy][9]) continue;
					tx=(float)ix/resxtexmag+texOffsetx;

					glNormal3fv(&data[ix][iy ][3]);//,data[ix][iy ][4],data[ix][iy ][5]);
					glTexCoord2f(tx,ty);  glVertex3f(data[ix][iy ][0],data[ix][iy ][1],data[ix][iy ][2]);
					
					glNormal3fv(&data[ix][iy2][3]);//,data[ix][iy2][4],data[ix][iy2][5]);
					glTexCoord2f(tx,ty2); glVertex3f(data[ix][iy2][0],data[ix][iy2][1],data[ix][iy2][2]);
				}
				glEnd();
		}
		glPopMatrix();
	unset(pref);
	return;
}

//------------------------------------------------------------------------------------

float Mesh::get(float x, float y) { // 0-1, 0-1 -> 0-1
	int ix, iy;
	float v=0, v0=0, xi,yi;

	if      (x<0) x=0;
	else if (x>1) x=1;
	if      (y<0) y=0;
	else if (y>1) y=1;

	ix=x*resx; 
	iy=y*resy;
	//if (ix>resx-2) ix=resx-2;
	//else if (ix<0) ix=0;
	//if (iy>resy-2) iy=resy-2;
	//else if (iy<0) iy=0;
	xi=x*resx-ix;
	yi=y*resy-iy;
	//if (xi==1) xi=0;
	//if (yi==1) yi=0;

	v0=data[ix][iy][1];
	v=v0;
	v+=(data[    (int)minv((ix+1),resx)][iy][1]-v0)*xi;
	v+=(data[ix][(int)minv((iy+1),resy)]    [1]-v0)*yi;
	return v;
}

//--------------------------------------------------------------------

void Mesh::getv(float x, float y, float *xi, float *zi) {
	*xi=-(get(x+.01,y    )-
		  get(x-.01,y    ));
	*zi=-(get(x,    y+.01)-
		  get(x    ,y-.01));
}

//--------------------------------------------------------------------

void Mesh::calc_normal(void) {
	int i, ix,iy, ix2,iy2;
	float x,y,z;
	Vector v1,v2,v3;

	for    (iy=0; iy<resy-1; iy++) { 
		iy2=iy+1;
		for (ix=0; ix<resx-1; ix++) {
			ix2=ix+1;
			x=data[ix][iy][0];
			y=data[ix][iy][1];
			z=data[ix][iy][2];
			v2.make(data[ix ][iy2][0]-x, data[ix ][iy2][1]-y, data[ix ][iy2][2]-z);
			v3.make(data[ix2][iy ][0]-x, data[ix2][iy ][1]-y, data[ix2][iy ][2]-z);
			v1=v2.cross_prod(v3);
			v1=v1.normalize();
			data[ix][iy][6]=v1.x;
			data[ix][iy][7]=v1.y;
			data[ix][iy][8]=v1.z;
		}
	}
	for (ix=0; ix<resx; ix++) {
		for (i=6; i<9; i++) {
			data[ix][resx-1][i]=
			data[ix][resx-2][i];
		}
	}
	for (iy=0; iy<resy; iy++) {
		for (i=6; i<9; i++) {
			data[resx-1][iy][i]=
			data[resx-2][iy][i];
		}
	}

	for    (iy=1; iy<resy-1; iy++) { 
		for (ix=1; ix<resx-1; ix++) {
			for (i=3; i<6; i++) {
				data[ix][iy][i]=(data[ix  ][iy  ][i+3]+
					              data[ix-1][iy  ][i+3]+
								  data[ix+1][iy  ][i+3]+
								  data[ix  ][iy-1][i+3]+
								  data[ix  ][iy+1][i+3])/5;
			}
		}
	}
	for (ix=0; ix<resx; ix++) {
		for (i=3; i<6; i++) {
			data[ix][     0][i]=data[ix][     0][i+3];
			data[ix][resy-1][i]=data[ix][resy-1][i+3];
		}
	}
	for (iy=0; iy<resy; iy++) {
		for (i=3; i<6; i++) {
			data[     0][iy][i]=data[     0][iy][i+3];
			data[resx-1][iy][i]=data[resx-1][iy][i+3];
		}
	}
	return;
}

//--------------------------------------------------------------------

void Mesh::calc_flat(void) {
	int ix,iy,iy2;
	float lh,lh2,nx,ny,nz,nx2,ny2,nz2, d=.00001;

	for    (iy=0; iy<resy-2; iy++) {
		iy2=iy+1;
		lh=999; lh2=999;
		nx=999; ny=999; nx=999;
		nx2=999;ny2=999;nz2=999;
		for (ix=0; ix<resx-1; ix++) {
			if ((lh ==data[ix+1][iy ][1])&&
				 (lh2==data[ix+1][iy2][1])&&
				 (fabs(nx-data[ix+1][iy][3])<d)&&
				 (fabs(ny-data[ix+1][iy][4])<d)&&
				 (fabs(nz-data[ix+1][iy][5])<d)&&
				 (fabs(nx2-data[ix+1][iy2][3])<d)&&
				 (fabs(ny2-data[ix+1][iy2][4])<d)&&
				 (fabs(nz2-data[ix+1][iy2][5])<d)&&
				 (ix!=resx-2)) data[ix][iy ][9]=1;
			else {
				data[ix][iy ][9]=0;
				lh =data[ix][iy ][1];
				lh2=data[ix][iy2][1];
				nx=data[ix][iy][3];
				ny=data[ix][iy][4];
				nz=data[ix][iy][5];
				nx2=data[ix][iy2][3];
				ny2=data[ix][iy2][4];
				nz2=data[ix][iy2][5];
			}
		}
	}
	return;
}

//---------------------------------------------------------------------------------------

void Mesh::draw_grass(Pref *pref) {
	int ix,iy,ix2,iy2;
	float resxtexmag, resytexmag;

	glPushMatrix();
		glTranslatef(x,y,z);
		resxtexmag=(resx-1)/texmagx;
		resytexmag=(resy-1)/texmagy;
		//if (setup) set(pref);
		glBegin(GL_QUADS);
		for         (iy=0; iy<resy; iy++) {
			float h=10*pref->page;
			iy2=iy+1;
			//ty =(float)iy /resytexmag+texOffsety;
			//ty2=(float)iy2/resytexmag+texOffsety;
				for (ix=0; ix<resx; ix++) {
					if (data[ix][iy][9]) continue;
					ix2=ix+1;
					//tx =(float)ix /resxtexmag+texOffsetx;
					//tx2=(float)ix2/resxtexmag+texOffsetx;

					glTexCoord2f(0,0);
					glVertex3fv(&data[ix][iy][0]);//,data[ix][iy ][1],data[ix][iy ][2]);
					
					glTexCoord2f(texmagx,0);
					glVertex3fv(&data[ix2][iy][0]);//,data[ix][iy2][1],data[ix][iy2][2]);

					glTexCoord2f(texmagx,1);
					glVertex3f(data[ix2][iy][0],data[ix2][iy][1]+h,data[ix2][iy][2]);

					glTexCoord2f(0,1);
					glVertex3f(data[ix][iy][0],data[ix][iy][1]+h,data[ix][iy][2]);
				}
			}
			glEnd();
		glPopMatrix();
	//if (setup) unset(pref);
	return;
}

//------------------------------------------------------------------------------------

float Mesh::rnd(float n) {
	return (rndn(size)-.5*size)*n;
}

//------------------------------------------------------------------------------------



