#if !defined(object_h)
#define      object_h

#include "main.h"
#include "counter.h"
#include "vector.h"
#include "snd.h"
//#include <iostream.h>
//#include <fstream.h>

//----------------------------------------------------------------------------------
class Camera;
class Pref;
class Text;
class Texture;
class Controller;
class Hardware;

class Object{
public:		Object();
	virtual ~Object();

	int on, origin, faceScreen, setup;
	float x,y,z, xi,yi,zi, size,sizex,sizey,sizez, r,g,b,a;
	float pitch,heading,roll,pitchi,headingi,rolli;
	float x0,y0,z0, x1,y1,z1, texmagx, texmagy;

	int   alphatest, blend,blendFunc0,blendFunc1, depthtest,depthwrite, lightmap;
	float alphafunc;
	Texture *tex;
	Controller *controller;
	static Hardware *hw;

	virtual void init_bounds(float x, float y, float z);
	virtual void move(float fDeltaTime, Pref *pref);
	void bounce(float fDeltaTime, Pref *pref);
	virtual int collision(Object *object, Pref *pref);
	virtual void draw(Camera *camera, Pref *pref);
	void setcolor(float inr, float ing, float inb, float ina=1);
	void set(Pref *pref);
	void unset(Pref *pref);
	void drag(float speed);
};

//------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------

class Key;
class Mouse;
class Time;

class Camera: public Object{
public:		Camera();
	virtual ~Camera();

	Counter *cameraMove, *cameraRotate;
	float  fov, shakeampl, jogx,jogy,jogz, speedLimit, center;
	float  forwardx,forwardy,forwardz, rightx,righty,rightz, upx,upy,upz;
	float  moveSpeed, rotateSpeed, maxthrust;
	float  moveRangex, moveRangey, moveRangez;
	float pitchRange, headingRange, rollRange;
	float translatex, translatey, translatez;
	float centerx,centery,centerz;
	Pref *pref;
	Time *time;
	Mouse *mouse;

	GLfloat model[16];

	int  frustum_check(Vector v);
	void shake(void);
	void get_matrix(void);
	void orient(void);
	void move_keys(Key *key);
	void kick(void);
};

//---------------------------------------------------------------------------------

class Key{
public:		Key();
	virtual ~Key();

	int key, skey, alt, control, shift, move, moved;
	float MoveSensitivity;

	void set(int key);
	void   set_move(int key);
	void unset_move(int key);

};

//---------------------------------------------------------------------------------

class Material{
public:		Material();
	virtual ~Material();

	float ambient, diffuse, specular, reflective, shininess;
};

//---------------------------------------------------------------------------------

class Light:public Object, public Material{
public:		Light();
	virtual ~Light();

	int   lightn, direction;
	float cutoff, exponent, inten, rotateSpeed;

	void set(void);
	void position(float x, float y, float z);
};

//---------------------------------------------------------------------------------

class Screen;

class Mouse{
public:		Mouse();
	virtual ~Mouse();

	int snapx,snapy;
	int x,y, x_cartesian,y_cartesian, x_center,y_center, x_exp,y_exp, lastx,lasty, mode, moved;
	int lbutton, mbutton, rbutton;
	float mouseSensitivity, xcart,ycart;

	void set_button(int button, int state);
	void set(int x, int y, Pref *pref, Screen *screen);
	void set_cursor(Pref *pref, int dialogs);
	void buttonstep(void);
};

//---------------------------------------------------------------------------------

class Screen{
public:		Screen();
	virtual ~Screen();

	int fullscreen, width,height, clipy0,clipy1, clipheight;
	float letterbox, colordepth, needblank;
	//GLfloat aspect;
	float nearz, farz, r,g,b, maskr,maskg,maskb;
	Pref *pref;
	Camera *camera;

	void clear(void);
	void color(float r, float g, float b);
	void set_letterbox(int);
	void set_viewport(void);
	void set_ortho(void);
	void set_perspective(float aspect);
	void fade(float alpha);
	void mask(void);
};

//---------------------------------------------------------------------------------

class Time{
public:		Time();
	virtual ~Time();

	float fTime, fDeltaTime, lastTime, baseTime, realTime;
	int   msTime;

	void update(void);
};

//---------------------------------------------------------------------------------
//---------------------------------------------------------------------------------
//---------------------------------------------------------------------------------

class Score{
public:		Score();
	virtual ~Score();

	float scores[19];
	int   dir, updated, lastLevelUpdated;

	void init(void);
	int  update(int level, float value);
	void display(void);
	void firstLoad(const char *titlestr);
	void finalSave(const char *titlestr);
	int  save(const char* title);
	int  load(const char* title);
};

//---------------------------------------------------------------------------------

class Pref{
public:		Pref();
	virtual ~Pref();

	//friend ostream& operator <<(ostream& os, Pref& pref);
	//friend istream& operator >>(istream& is, Pref& pref);

	int    fullscreen, letterbox, tess, universe, blank, lights, textures, wireframe;
	int    view, option, image, fclock, map, fog, n, n2, blendFunc0, blendFunc1, pause;
	int toggle[9], arrownx,arrowny, windowx, windowy, lightmap, reverbMode;
	float  fov, spd1,spd2, brace,bracket, arrowx,arrowy, page, home, a1,a2,b1,b2,c1,c2, paren;
	float  mouseSens, moveSens, volume, pan;
	float lmtextureMagx, lmtextureMagy, lmInterval, lmnoise;

	float  r,g,b,a, brightness, ambient, diffuse, specular, reflective, shininess, cutoff, exponent;
	float value[9];
	int load(const char* title);
	int save(const char* title);
};

//---------------------------------------------------------------------------------

class Matrix{
public:		Matrix();
	virtual ~Matrix();

	GLfloat m[16];
	float	pitch, heading, roll;


	Vector rotatev(Vector);
	void v2m(Camera *camera);
	void set_rot_axis(Vector axis, float value);

};

//---------------------------------------------------------------------------------
class Snd;
class Hardware{
public:		Hardware();
	virtual ~Hardware();

	Camera *camera;
	Counter *counter;
	Key *key;
	Mouse *mouse;
	Pref *pref;
	Snd *sound;
	Screen *screen;
	Text *gen;
	Time *time;
};

//---------------------------------------------------------------------------------
/*
enum inputs {MOUSEX,MOUSEY, ARROWX,ARROWY,
             ARROWXDOWN,ARROWYDOWN,
             LEFTBUTTON,MIDDLEBUTTON,RIGHTBUTTON, AUTO};

class Controller{
public:		Controller();
	virtual ~Controller();

	int x,y,z, xi,yi,zi, pitch,heading,roll;

	float  scalex, scaley, scalez,  scalexi, scaleyi, scalezi,  scalepitch, scaleheading, scaleroll;
	float offsetx,offsety,offsetz, offsetxi,offsetyi,offsetzi, offsetpitch,offsetheading,offsetroll;

	int counterx, countery, counterz, counterpitch, counterheading, counterroll;
	float speedx,   speedy,   speedz,   speedpitch,   speedheading,   speedroll;

	static Hardware *hw;
	Counter *counter;

	void control(Object object);
};

//---------------------------------------------------------------------------------
*/

#endif