//-------------------------------------------------------------------
//	File:		   particle.c
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "particle.h"

//--------------------------------------------------------------------------------------

Particle::Particle() {
	maxparticles=100;
	kill();
	windx=0; windy=0; windz=0;
	pitch=0; heading=0; roll=0; rollspeed=1;
	pitchi=0; headingi=0; rolli=0;
	size0=.1; size1=.1; dispersex=0; dispersey=0; dispersez=0; drag=1; bouncey=0;
	lifeTime=1;	interval=.1;
	inten0=1, inten1=0; blendFunc0=2; blendFunc1=1;
	alpha0=1; alpha1=1;
	r0=1; g0=1; b0=1;
	r1=1; g1=0; b1=0; colorRnd=0; sizeRnd=0; lifeTimeRnd=0;
	alphatest=1; alphafunc=.1; blend=1; depthtest=1; depthwrite=0;
	tx0=0; tx1=1; ty0=0; ty1=1; 
	tex=0;  canroll=0;
}

Particle::~Particle() {};

Camera *Particle::camera=0;
Pref   *Particle::pref=0;
Time   *Particle::time=0;

//--------------------------------------------------------------------------------------

int Particle::create(float x, float y, float z) {
	int i, i2, found=-1;
	static int current=0;
	float dispersexh,disperseyh,dispersezh;

	if (time->fTime<(lastTime+interval)) return found;

	for (i=current; i<maxparticles+current; i++) {
		i2=i%maxparticles;
		if (p[i2].on==0) {
			found=i2;
			current=(i2+1)%maxparticles;
			break;
		}
	}
	if (found>-1) {
		float r,g,b;

		dispersexh=.5*dispersex;
		disperseyh=.5*dispersey;
		dispersezh=.5*dispersez;
		lastTime=time->fTime;
		p[found].on=1;
		p[found].x=x; p[found].lastx=x;
		p[found].y=y; p[found].lasty=y;
		p[found].z=z; p[found].lastz=z;
		p[found].xi= rndn(dispersex)-dispersexh+xi;
		p[found].yi= rndn(dispersey)-disperseyh+yi;
		p[found].zi= rndn(dispersez)-dispersezh+zi;
		p[found].pitch  =0;
		p[found].heading=0;
		p[found].pitchi  =pitchi;
		p[found].headingi=headingi;

		if (canroll) {
			p[found].roll =rndn(360);
			p[found].rolli=(rndn(2)-1)*rollspeed;
		} else {
			p[found].roll =0;
			p[found].rolli=0;
		}
		p[found].size =1+(rndn(1)-.5)*sizeRnd;
		r=r0+rndn(colorRnd)-.5*colorRnd; r=(r<0)?0:(r>1)?1:r; p[found].r0=r;
		g=g0+rndn(colorRnd)-.5*colorRnd; g=(g<0)?0:(g>1)?1:g; p[found].g0=g;
		b=b0+rndn(colorRnd)-.5*colorRnd; b=(b<0)?0:(b>1)?1:b; p[found].b0=b;
		r=r1+rndn(colorRnd)-.5*colorRnd; r=(r<0)?0:(r>1)?1:r; p[found].r1=r;
		g=g1+rndn(colorRnd)-.5*colorRnd; g=(g<0)?0:(g>1)?1:g; p[found].g1=g;
		b=b1+rndn(colorRnd)-.5*colorRnd; b=(b<0)?0:(b>1)?1:b; p[found].b1=b;

		p[found].birthTime=time->fTime;
		p[found].lifeTime=lifeTime+(rndn(1)-.5)*lifeTimeRnd*lifeTime;
		p[found].deathTime=time->fTime+p[found].lifeTime;
		p[found].slickness=1;
	}
	return found;
}

//-----------------------------------------------------------------------

void Particle::update(void) {
	int i;
	float inc;

	inc=time->fDeltaTime;
	for (i=0; i<maxparticles; i++) {
		if (p[i].on==0) continue;
		if ((p[i].lifeTime>0)&&(time->fTime>p[i].deathTime)) {p[i].on=0; continue;}
		p[i].x+=p[i].xi*inc;
		p[i].y+=p[i].yi*inc;
		p[i].z+=p[i].zi*inc;

		p[i].xi+=(windx*time->fDeltaTime)*p[i].slickness;
		p[i].yi+=(windy*time->fDeltaTime)*p[i].slickness;
		p[i].zi+=(windz*time->fDeltaTime)*p[i].slickness;

		p[i].pitch  +=p[i].pitchi  *time->fDeltaTime;
		p[i].heading+=p[i].headingi*time->fDeltaTime;
		if (canroll)  p[i].roll+=p[i].rolli*time->fDeltaTime;
	}
	return;
}

//---------------------------------------------------------------------

void Particle::draw(void) {
	float size, inten, age, alpha;
	int i;

	set();
	if (tex) { // textured quad
		glEnable(GL_TEXTURE_2D);
		tex->bind();

		for (i=0; i<maxparticles; i++) {
			float rightxs,rightys,rightzs, upxs,upys,upzs;

			if (!p[i].on) continue;
			if (p[i].lifeTime>0) {
				age=(time->fTime-p[i].birthTime)/p[i].lifeTime;
				if (age>1) age=0;
				size =lin_interp(size0,size1,age)*p[i].size;//*pref->bracket;
				if (size<=0) size=size0;
				alpha=lin_interp(alpha0,alpha1,age);
				inten=cos_interp(inten0,inten1, age);
				glColor4f(lin_interp(p[i].r0,p[i].r1,age)*inten,
						    lin_interp(p[i].g0,p[i].g1,age)*inten,
						    lin_interp(p[i].b0,p[i].b1,age)*inten, alpha);
			}
			else {
				age=0;
				size=size0*p[i].size;//*pref->bracket;
				inten=inten0;
				glColor4f(p[i].r0*inten, p[i].g0*inten, p[i].b0*inten, alpha0);
			}
			if (!canroll) {
				rightxs=camera->rightx*p[i].size*size;
				rightys=camera->righty*p[i].size*size;
				rightzs=camera->rightz*p[i].size*size;
				upxs=camera->upx*p[i].size*size;
				upys=camera->upy*p[i].size*size;
				upzs=camera->upz*p[i].size*size;
			}
			glPushMatrix();
				glTranslatef(p[i].x+x,p[i].y+y,p[i].z+z);
				if (canroll) {
					glRotatef(-camera->heading-p[i].heading, 0,1,0);
					glRotatef(-camera->pitch  -p[i].pitch,   1,0,0);
					glRotatef(-camera->roll   -p[i].roll,    0,0,1);
					glBegin(GL_QUADS);
						glTexCoord2f(tx0,ty0); glVertex2f(-size,-size);
						glTexCoord2f(tx1,ty0); glVertex2f( size,-size);
						glTexCoord2f(tx1,ty1); glVertex2f( size, size);
						glTexCoord2f(tx0,ty1); glVertex2f(-size ,size);
						glEnd();
				} else {
					glBegin(GL_QUADS);
						glTexCoord2f(tx0,ty0); glVertex3f(-rightxs-upxs,-rightys-upys,-rightzs-upzs);
						glTexCoord2f(tx1,ty0); glVertex3f( rightxs-upxs, rightys-upys, rightzs-upzs);
						glTexCoord2f(tx1,ty1); glVertex3f( rightxs+upxs, rightys+upys, rightzs+upzs);
						glTexCoord2f(tx0,ty1); glVertex3f(-rightxs+upxs,-rightys+upys,-rightzs+upzs);
						glEnd();
				}
				glPopMatrix();
		}
		if (tex) glDisable(GL_TEXTURE_2D);
	}
	else { // glpoint
		for (i=0; i<maxparticles; i++) {
			if (!p[i].on) continue;
			if (p[i].lifeTime>0) {
				age=(time->fTime-p[i].birthTime)/p[i].lifeTime;
				alpha=lin_interp(alpha0,alpha1,age);
				inten=cos_interp(inten0,inten1, age);
				size=lin_interp(size0,size1, age)*p[i].size;
				glColor4f(lin_interp(p[i].r0,p[i].r1,age)*inten,
						    lin_interp(p[i].g0,p[i].g1,age)*inten,
						    lin_interp(p[i].b0,p[i].b1,age)*inten, alpha);
			} else {
				inten=inten0;
				glColor4f(p[i].r0*inten, p[i].g0*inten, p[i].b0*inten, alpha0);
			}
			glPushMatrix();
				glTranslatef(p[i].x+x,p[i].y+y,p[i].z+z);
				//size=(size<1)?1:(size>9)?9:size;
				if (length==0) {
					glPointSize((float)size);
					//glPointSize(8);
					glBegin(GL_POINTS);
						glVertex3d(0,0,0);
						glEnd();
				} else {
					glLineWidth((float)size);
					glBegin(GL_LINES);
						float v,l;

						v=sqrt(p[i].xi*p[i].xi+
							   p[i].yi*p[i].yi+
							   p[i].zi*p[i].zi);
						if (v) l=1./v; else l=0;
						l=l*length;
						glVertex3d(0,0,0);
						glVertex3d(p[i].xi*l,p[i].yi*l,p[i].zi*l);
						glEnd();
				}
				glPopMatrix();
		}
	}
	unset();
	return;
}

//---------------------------------------------------------------------

void Particle::set(void) {
	int bfa[]={GL_ZERO,GL_ONE,
		      GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA,
			  GL_DST_ALPHA,GL_ONE_MINUS_DST_ALPHA,
			  GL_DST_COLOR,GL_ONE_MINUS_DST_COLOR,
			  GL_SRC_ALPHA_SATURATE};
	int bfb[]={GL_ZERO,GL_ONE,
		      GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA,
			  GL_DST_ALPHA,GL_ONE_MINUS_DST_ALPHA,
			  GL_SRC_COLOR,GL_ONE_MINUS_SRC_COLOR};

	if (alphatest) {
		glAlphaFunc(GL_GREATER,alphafunc);
		glEnable(GL_ALPHA_TEST);
	}
	if (blend) {
		glBlendFunc(bfa[blendFunc0],bfb[blendFunc1]);
		glEnable(GL_BLEND);
	}
	if (depthtest) {
		if (!depthwrite) glDepthMask(GL_FALSE);
		glEnable(GL_DEPTH_TEST);
	}
	return;
}

//---------------------------------------------------------------------

void Particle::unset(void) {
	if (alphatest) {
		glDisable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER,0);
	}
	if (blend) {
		glDisable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	if (depthtest) {
		glDisable(GL_DEPTH_TEST);
		if (!depthwrite) glDepthMask(GL_TRUE);
	}
	return;
}

//---------------------------------------------------------------------

void Particle::kill(void) {
	int i;
	for (i=0; i<maxparticles; i++) p[i].on=0;
	init=0;
	return;
}

//---------------------------------------------------------------------

int Particle::count(void) {
	int i, count=0;
	for (i=0; i<maxparticles; i++) if (p[i].on) count++;
	return count;
}

//---------------------------------------------------------------------

int Particle::collision(float x, float y, float z, float distance) {
	int i;

	for (i=0; i<maxparticles; i++) {
		if (p[i].on==0) continue;
		if ( (fabs(p[i].x-x)<distance)&&
			  (fabs(p[i].y-y)<distance)&&
			  (fabs(p[i].z-z)<distance) ) { p[i].on=0; return 1; }
	}
	return 0;
}

//----------------------------------------------------------------------

